#include <cmath>
#include <stdexcept>

#include "npstat/nm/sineTransformMatrix.hh"

#define PI 3.1415926535897932385
#define SQR2 1.4142135623730950488

namespace npstat {
    Matrix<double> sineTransformMatrix(const unsigned N, const unsigned K)
    {
        if (!N) throw std::invalid_argument(
            "In npstat::sineTransformMatrix: "
            "number of sine terms must be positive");
        if (!K) throw std::invalid_argument(
            "In npstat::sineTransformMatrix: "
            "number of discretization points must be positive");

        const double h = 1.0/K;
        const double norm = SQR2*h;

        npstat::Matrix<double> result(N, K);
        for (unsigned n=1; n<=N; ++n)
        {
            const double nPih = n*PI*h;
            double* base = result[n - 1U];
            for (unsigned j=0; j<K; ++j)
                base[j] = norm*sin(nPih*(j + 0.5));
        }

        return result;
    }
}
