#ifndef NPSTAT_SINETRANSFORMMATRIX_HH_
#define NPSTAT_SINETRANSFORMMATRIX_HH_

/*!
// \file sineTransformMatrix.hh
//
// \brief Matrix for the simple sine transform on the [0, 1] interval
//
// Author: I. Volobouev
//
// October 2022
*/

#include "npstat/nm/Matrix.hh"

namespace npstat {
    /**
    // nTerms     is the number of sine terms in the transform. The basis
    //            functions are sqrt(2)*sin(k*Pi*x), k = 1, 2, ..., nTerms.
    //
    // nDiscrete  is the number of points to use for discretizing the [0, 1]
    //            interval. Should normally be substantially larger than
    //            "nTerms".
    //
    // The returned matrix will have nTerms rows and nDiscrete columns.
    */
    Matrix<double> sineTransformMatrix(unsigned nTerms, unsigned nDiscrete);
}

#endif // NPSTAT_SINETRANSFORMMATRIX_HH_
