#include <sstream>
#include <algorithm>
#include <stdexcept>

#include "npstat/nm/sineTransformMatrix.hh"

#include "npstat/stat/SineGOFTest1D.hh"
#include "npstat/stat/Distributions1D.hh"

namespace npstat {
    SineGOFTest1D::SineGOFTest1D(
        const AbsDistribution1D& nullDistro,
        const unsigned nSineTerms, const unsigned nDiscretizationPoints,
        const unsigned char* mask, const unsigned lenMask)
        : AbsSmoothGOFTest1D(nullDistro, nSineTerms, mask, lenMask),
          nPt_(nDiscretizationPoints),
          sineM_(sineTransformMatrix(nSineTerms, nDiscretizationPoints)),
          points_(0.0, 1.0, nDiscretizationPoints),
          quantiles_(nDiscretizationPoints),
          cdfValues_(nDiscretizationPoints),
          sineValues_(nSineTerms)
    {
        for (unsigned icdf=0; icdf<nPt_; ++icdf)
            quantiles_[icdf] = distro_->quantile(points_[icdf]);
    }

    SineGOFTest1D::SineGOFTest1D(const AbsDistribution1D& nullDistro,
                                 const unsigned nDiscretizationPoints,
                                 const std::vector<int>& mask)
        : AbsSmoothGOFTest1D(nullDistro, mask),
          nPt_(nDiscretizationPoints),
          sineM_(sineTransformMatrix(mask.size(), nDiscretizationPoints)),
          points_(0.0, 1.0, nDiscretizationPoints),
          quantiles_(nDiscretizationPoints),
          cdfValues_(nDiscretizationPoints),
          sineValues_(mask.size())
    {
        for (unsigned icdf=0; icdf<nPt_; ++icdf)
            quantiles_[icdf] = distro_->quantile(points_[icdf]);
    }

    unsigned SineGOFTest1D::normalizedDeviations(
        const double* data, const unsigned long lenData,
        const bool dataIsSorted,
        double* deviations, const unsigned lenDeviations) const
    {
        assert(lenData);
        assert(data);
        assert(lenDeviations >= nUnmasked_);

        if (dataIsSorted)
            return normalizedDeviationsSorted(data, lenData, deviations);
        else
        {
            std::vector<double> tmp(data, data+lenData);
            std::sort(tmp.begin(), tmp.end());
            return normalizedDeviationsSorted(&tmp[0], lenData, deviations);
        }
    }

    unsigned SineGOFTest1D::normalizedDeviations(
        const float* data, const unsigned long lenData,
        const bool dataIsSorted,
        double* deviations, const unsigned lenDeviations) const
    {
        assert(lenData);
        assert(data);
        assert(lenDeviations >= nUnmasked_);

        if (dataIsSorted)
            return normalizedDeviationsSorted(data, lenData, deviations);
        else
        {
            std::vector<float> tmp(data, data+lenData);
            std::sort(tmp.begin(), tmp.end());
            return normalizedDeviationsSorted(&tmp[0], lenData, deviations);
        }
    }
}
