#ifndef NPSTAT_SINEGOFTEST1D_HH_
#define NPSTAT_SINEGOFTEST1D_HH_

/*!
// \file SineGOFTest1D.hh
//
// \brief Goodness-of-fit test based on the sine expansion of the
//        difference between the empirical cdf and the cdf of the null
//
// Author: I. Volobouev
//
// October 2022
*/

#include "npstat/nm/Matrix.hh"
#include "npstat/nm/EquidistantSequence.hh"

#include "npstat/stat/AbsSmoothGOFTest1D.hh"

namespace npstat {
    class SineGOFTest1D : public AbsSmoothGOFTest1D
    {
    public:
        // The mask, if provided, should contain nSineTerms elements.
        // Put some number in the mask in order to discard the
        // corresponding eigenvector and put 0 there to keep it.
        // The sine terms (sin(Pi n x)) numbering in the mask
        // is shifted: n = 1 corresponds to mask[0], n = 2 corresponds
        // to mask[1], etc.
        SineGOFTest1D(const AbsDistribution1D& nullDistro,
                      unsigned nSineTerms, unsigned nDiscretizationPoints,
                      const unsigned char* mask=0, unsigned lenMask=0);

        SineGOFTest1D(const AbsDistribution1D& nullDistro,
                      unsigned nDiscretizationPoints,
                      const std::vector<int>& mask);

        inline virtual ~SineGOFTest1D() {}

        inline virtual std::string shortName() const
            {return testNameWithMask("Sine");}

        virtual unsigned normalizedDeviations(
            const double* data, unsigned long sz,
            bool isDataSorted,
            double* deviations, unsigned lenDeviations) const;

        virtual unsigned normalizedDeviations(
            const float* data, unsigned long sz,
            bool isDataSorted,
            double* deviations, unsigned lenDeviations) const;

    private:
        template<typename Numeric>
        unsigned normalizedDeviationsSorted(
            const Numeric* data, unsigned long lenData,
            double* deviations) const;

        unsigned nPt_;
        Matrix<double> sineM_;
        EquidistantBinCenters points_;
        std::vector<double> quantiles_;
        mutable std::vector<double> cdfValues_;
        mutable std::vector<double> sineValues_;
    };
}

#include "npstat/stat/SineGOFTest1D.icc"

#endif // NPSTAT_SINEGOFTEST1D_HH_
