#ifndef NPSTAT_SMOOTHCDGOFTEST1D_HH_
#define NPSTAT_SMOOTHCDGOFTEST1D_HH_

/*!
// \file SmoothCDGOFTest1D.hh
//
// \brief Smooth test for goodness-of-fit that utilizes the empirical
//        comparison density
//
// Author: I. Volobouev
//
// October 2022
*/

#include "npstat/nm/ClassicalOrthoPolys1D.hh"
#include "npstat/stat/AbsSmoothGOFTest1D.hh"

namespace npstat {
    class SmoothCDGOFTest1D : public AbsSmoothGOFTest1D
    {
    public:
        // The mask, if provided, should contain "maxdeg" elements.
        // Put some number in the mask in order to discard the
        // corresponding eigenvector and put 0 there to keep it.
        // mask[0] specifies whether to use polynomial of degree 1, etc.
        inline SmoothCDGOFTest1D(const AbsDistribution1D& distro,
                                 const unsigned maxdeg,
                                 const unsigned char* mask=0,
                                 const unsigned lenMask=0)
            : AbsSmoothGOFTest1D(distro, maxdeg, mask, lenMask) {}

        inline SmoothCDGOFTest1D(const AbsDistribution1D& distro,
                                 const std::vector<int>& mask)
            : AbsSmoothGOFTest1D(distro, mask) {}

        inline virtual ~SmoothCDGOFTest1D() {}

        inline virtual std::string shortName() const
            {return testNameWithMask("SCD");}

        virtual unsigned normalizedDeviations(
            const double* data, unsigned long sz,
            bool isDataSorted,
            double* deviations, unsigned lenDeviations) const;

        virtual unsigned normalizedDeviations(
            const float* data, unsigned long sz,
            bool isDataSorted,
            double* deviations, unsigned lenDeviations) const;

    private:
        template<typename Numeric>
        unsigned normalizedDeviationsHelper(
            const Numeric* data, unsigned long lenData,
            double* deviations) const;

        ShiftedLegendreOrthoPoly1D poly_;
    };
}

#include "npstat/stat/SmoothCDGOFTest1D.icc"

#endif // NPSTAT_SMOOTHCDGOFTEST1D_HH_
