#include <cassert>

#include "npstat/stat/TwoDistros1DFunctors.hh"

#include "npstat/nm/ClassicalOrthoPolys1D.hh"
#include "npstat/nm/GaussLegendreQuadrature.hh"

namespace {
    class ComparisonDensityProdFcn : public npstat::Functor1<double, double>
    {
    public:
        inline ComparisonDensityProdFcn(
            const npstat::ComparisonDensityFunctor1D& cd, const unsigned deg)
            : cd_(cd), deg_(deg) {}

        inline virtual ~ComparisonDensityProdFcn() {}

        inline virtual double operator()(const double& x) const
            {return cd_(x)*poly_.poly(deg_, x);}

    private:
        const npstat::ComparisonDensityFunctor1D& cd_;
        npstat::ShiftedLegendreOrthoPoly1D poly_;
        unsigned deg_;
    };
}

namespace npstat {
    double ComparisonDensityFunctor1D::legendreCoeff(
        const unsigned degree) const
    {
        GaussLegendreQuadrature glq(1024U);
        ComparisonDensityProdFcn fcn(*this, degree);
        return glq.integrate(fcn, 0.0, 1.0);
    }

    void ComparisonDensityFunctor1D::legendreSeriesCoeffs(
        double* coeffs, const unsigned maxDeg) const
    {
        assert(coeffs);

        ShiftedLegendreOrthoPoly1D poly;
        GaussLegendreQuadrature glq(1024U);
        poly.extWeightAverages(*this, glq, coeffs, maxDeg);
    }
}
