#ifndef NPSTAT_TWODISTROS1DFUNCTORS_HH_
#define NPSTAT_TWODISTROS1DFUNCTORS_HH_

/*!
// \file TwoDistros1DFunctors.hh
//
// \brief A few functors used to study GoF tests
//
// Author: I. Volobouev
//
// October 2022
*/

#include "npstat/stat/AbsTwoDistros1DFunctor.hh"

namespace npstat {
    class CdfDeltaFunctor1D : public AbsTwoDistros1DFunctor
    {
    public:
        inline CdfDeltaFunctor1D(const AbsDistribution1D& nullHypothesis,
                                 const AbsDistribution1D& alternative)
            : AbsTwoDistros1DFunctor(nullHypothesis, alternative) {}

        inline virtual ~CdfDeltaFunctor1D() {}

        inline virtual double operator()(const double& x) const
        {
            if (x < 0.0 || x > 1.0)
                return 0.0;
            else
                return altPtr_->cdf(nullPtr_->quantile(x)) - x;
        }
    };

    class ComparisonDensityFunctor1D : public AbsTwoDistros1DFunctor
    {
    public:
        inline ComparisonDensityFunctor1D(
            const AbsDistribution1D& nullHypothesis,
            const AbsDistribution1D& alternative)
            : AbsTwoDistros1DFunctor(nullHypothesis, alternative) {}

        inline virtual ~ComparisonDensityFunctor1D() {}

        inline virtual double operator()(const double& x) const
        {
            if (x < 0.0 || x > 1.0)
                return 0.0;
            else
            {
                const double y = nullPtr_->quantile(x);
                return altPtr_->density(y)/nullPtr_->density(y);
            }
        }

        /**
        // Coefficient of expansion of this function into
        // shifted Legendre polynomials (that is, Legendre
        // polynomials orthonormal on [0, 1]).
        */
        double legendreCoeff(unsigned degree) const;

        /**
        // All coefficients of the Legendre polynomial expansion
        // of the density, up to the degree requested. The array
        // "coeffs" should have at least maxDeg+1 elements.
        */
        void legendreSeriesCoeffs(double* coeffs, unsigned maxDeg) const;

#ifdef SWIG
    public:
        inline std::vector<double>
        legendreSeriesCoeffs2(const unsigned maxDeg) const
        {
            std::vector<double> tmp(maxDeg+1U);
            legendreSeriesCoeffs(&tmp[0], maxDeg);
            return tmp;
        }
#endif // SWIG
    };
}

#endif // NPSTAT_TWODISTROS1DFUNCTORS_HH_
