#ifndef NPSTAT_POLYEXPECTATION_HH_
#define NPSTAT_POLYEXPECTATION_HH_

/*!
// \file polyExpectation.hh
//
// \brief Calculate expectation value of some polynomial w.r.t.
//        a density function
//
// Author: I. Volobouev
//
// October 2022
*/

#include "npstat/stat/AbsDistribution1D.hh"

namespace npstat {
    // Using nIntegrationPoints = 256 default because at the
    // moment this is the highest number of points supported
    // by the GaussHermiteQuadrature class. Currently, "Poly"
    // should be one of AbsClassicalOrthoPoly1D, ContOrthoPoly1D,
    // or ScalableClassicalOrthoPoly1D.
    //
    // The expectation is calculated on the support interval
    // of the distribution. If this interval is different from
    // the support interval of the polynomial system, results
    // returned by this function can be substantially different
    // from the results obtained with the "extWeightAverages"
    // method of the polynomial system (with the weight set to
    // the corresponding density functor).
    //
    template<class Poly>
    double polyExpectation(const AbsDistribution1D& distro,
                           const Poly& poly,
                           unsigned degree,
                           unsigned nIntegrationPoints = 256U,
                           bool useFejerQuadrature = false);
}

#include "npstat/stat/polyExpectation.icc"

#endif // NPSTAT_POLYEXPECTATION_HH_
