#ifndef NPSTAT_SYMEIGEN_HH_
#define NPSTAT_SYMEIGEN_HH_

#include <cassert>
#include <stdexcept>

#include "npstat/wrap/NumpyTypecode.hh"
#include "npstat/wrap/pyWrapUtils.hh"

#include "npstat/nm/Matrix.hh"
#include "npstat/nm/allocators.hh"

namespace npstat {
    PyObject* symEigen(const Matrix<double>& mat, EigenMethod m = EIGEN_SIMPLE)
    {
        static const int typenum = NumpyTypecode<double>::code;

        const unsigned nRows = mat.nRows();
        if (!nRows) throw std::invalid_argument(
            "In npstat::symEigen: uninitialized matrix");
        if (!mat.isSquare()) throw std::invalid_argument(
            "In npstat::symEigen: matrix is not square");

        npy_intp sh_eval[1];
        sh_eval[0] = nRows;
        PyObject* arr_eval = PyArray_SimpleNew(1, sh_eval, typenum);
        assert(arr_eval);
        DecrefHandle h_eval(arr_eval);

        npy_intp sh_evec[2];
        sh_evec[0] = nRows;
        sh_evec[1] = nRows;
        PyObject* arr_evec = PyArray_SimpleNew(2, sh_evec, typenum);
        assert(arr_evec);
        DecrefHandle h_evec(arr_evec);

        {
            Matrix<double> eigenvectors(nRows, nRows);
            mat.symEigen((double *)array_data(arr_eval), nRows, &eigenvectors, m);
            copyBuffer((double*)array_data(arr_evec), eigenvectors.data(), eigenvectors.length());
        }

        PyObject* tup = PyTuple_New(2);
        assert(tup);
        PyTuple_SET_ITEM(tup, 0, h_eval.release());
        PyTuple_SET_ITEM(tup, 1, h_evec.release());
        return tup;
    }
}

#endif // NPSTAT_SYMEIGEN_HH_
